<?php

require_once getShopBasePath() . 'modules/girocheckout/lib/GiroCheckout_SDK/GiroCheckout_SDK.php';

/**
 * Class girocheckout__order
 */
class girocheckout__order extends girocheckout__order_parent
{
  const CART_TYPE_PHYSICAL = "PHYSICAL";
  const CART_TYPE_DIGITAL = "DIGITAL";
  const CART_TYPE_MIXED = "MIXED";

  /**
   * Returns array of payment methods by GiroCheckout
   *
   * @return array $aGCPaymentMethods
   */
  protected $aGCPaymentMethods = array(
    'gc_creditcard' => 'gc_creditcard',
    'gc_giropay' => 'gc_giropay',
    'gc_eps' => 'gc_eps',
    'gc_directdebit' => 'gc_directdebit',
    'gc_ideal' => 'gc_ideal',
    'gc_paydirekt' => 'gc_paydirekt',
    'gc_sofortuw' => 'gc_sofortuw',
    'gc_maestro' => 'gc_maestro',
    'gc_bluecode' => 'gc_bluecode');

  /**
   * @param int $iSuccess
   *
   * @return string
   */
  protected function _getNextStep($iSuccess) {
    // get current language
    $sActualPayment = $this->getPayment()->oxpayments__oxid->value;
    $oOrder = oxNew('oxorder');
    $sess_challenge = oxRegistry::getSession()->getVariable("sess_challenge");

    if ($oOrder->load($sess_challenge) && in_array($sActualPayment, $this->aGCPaymentMethods)) {
      if (is_numeric($iSuccess) && $iSuccess >= 1) {
        if ($iSuccess === oxOrder::ORDER_STATE_ORDEREXISTS && !$oOrder->girocheckout__isPaymentDone()) {
          list($oOrder, $iSuccess) = $this->girocheckout__recreateOrder($oOrder);
        }
        $oBasket = $this->getBasket();
        $dAmount = $oBasket->getPrice()->getBruttoPrice(); //amount
        // get currency
        $oCur = $oBasket->getBasketCurrency();
        $sCur = $oCur->name;

        $iOrderId = $oOrder->oxorder__oxid->value; //orderid (char 32)

        $lang = oxRegistry::getLang()->getLanguageAbbr(); //lang

        //check supported languages.
        $supportedLangs = array(
          "de",
          "en",
          "es",
          "fr",
          "it",
          "ja",
          "pt",
          "nl",
          "cs",
          "sv",
          "da",
          "pl",
          "spde",
          "spen",
          "de_DE_stadtn"
        );

        if (!in_array($lang, $supportedLangs)) {
          $lang = "en";
        }
        else {
        }

        //get version of the plugin and the active shop.
        $sourceParam = $this->getGcSource();

        // generate transaction id to identify the transaction on notify and redirect
        $transactionId = oxUtilsObject::getInstance()->generateUID();

        $sStoken = $this->getSession()->getSessionChallengeToken();
        $sRtoken = $this->getSession()->getRemoteAccessToken(true);
        $urlRedirect = $this->getConfig()->getSslShopUrl() . 'index.php?cl=order&fnc=processGiroCheckoutRedirect&pm='
          . $this->getPayment()->getId() . '&sess_challenge=' . oxRegistry::getSession()->getVariable('sess_challenge')
          . '&' . $this->getSession()->getName() . '=' . $this->getSession()->getId() . '&stoken=' . $sStoken
          . '&rtoken=' . $sRtoken;
        $urlNotify = $this->getConfig()->getSslShopUrl() . 'index.php?cl=order&fnc=processGiroCheckoutNotify&pm='
          . $this->getPayment()->getId() . '&sess_challenge=' . oxRegistry::getSession()->getVariable('sess_challenge')
          . '&' . $this->getSession()->getName() . '=' . $this->getSession()->getId() . '&stoken=' . $sStoken
          . '&rtoken=' . $sRtoken;


        $oUser = $this->getUser();
        $iUserId = $oUser->oxuser__oxid->value; //customer id (char 32)
        $iUserNr = $oUser->oxuser__oxcustnr->value;
        // check that this is a creditcard payment
        if ($this->getPayment()->getId() == "gc_creditcard") {
          // change order status
          $oOrder->oxorder__oxtransstatus = new oxField('IN PAYMENT');
          $oOrder->oxorder__giroconnect__transaction_id = new oxField($transactionId);
          $oOrder->setPayment($oBasket->getPaymentId());
          $oOrder->save();
          $addressMatch = false;
          $aShippingData = $this->getShippingData($oOrder);

          if (empty($oOrder->oxorder__oxdellname->value)) {
            $addressMatch = true;
          }

          $tds2Address = $aShippingData["shippingStreet"];
          $tds2Postcode = $aShippingData["shippingZipCode"];
          $tds2City = $aShippingData["shippingCity"];
          $tds2Country = $aShippingData["shippingCountryIso"];

          $aTdsOptionalInfo = new \stdClass;
          $aTdsOptionalInfo->email = $oOrder->oxorder__oxbillemail->value;
          $aTdsOptionalInfo->addressesMatch = "true";

          if (!$addressMatch) {
            $tds2Address = $oOrder->oxorder__oxbillstreet->value . " " . $oOrder->oxorder__oxbillstreetnr->value;
            $tds2Postcode = $oOrder->oxorder__oxbillzip->value;
            $tds2City = $oOrder->oxorder__oxbillcity->value;
            $tds2Country = $aShippingData["shippingCountryIso"];
            $aTdsOptionalInfo->addressesMatch = "false";

            if (!empty($aShippingData["shippingStreet"]) && !empty($aShippingData["shippingZipCode"])
              && !empty($aShippingData["shippingCity"]) && !empty($aShippingData["shippingCountryIso"])
            ) {
              $aTdsOptionalInfo->shippingAddress = new \stdClass;
              $aTdsOptionalInfo->shippingAddress->line1 = $aShippingData["shippingStreet"];
              $aTdsOptionalInfo->shippingAddress->postcode = $aShippingData["shippingZipCode"];
              $aTdsOptionalInfo->shippingAddress->city = $aShippingData["shippingCity"];
              $aTdsOptionalInfo->shippingAddress->country = $aShippingData["shippingCountryIso"];
            }
          }

          try {
            $sPurpose = $this->getPurpose(oxRegistry::getConfig()->getConfigParam('GC_CREDITCARD_PURPOSE'), $oOrder, $oUser);

            //Sends request to Girocheckout.
            $request = new GiroCheckout_SDK_Request('creditCardTransaction');
            $request->setSecret(oxRegistry::getConfig()->getConfigParam('GC_CREDITCARD_SECRET'));
            $request->addParam('merchantId', oxRegistry::getConfig()->getConfigParam('GC_CREDITCARD_MERCHANTID'))
              ->addParam('projectId', oxRegistry::getConfig()->getConfigParam('GC_CREDITCARD_PROJECTID'))
              ->addParam('merchantTxId', $transactionId)
              ->addParam('amount', round($dAmount * 100))
              ->addParam('currency', strtoupper($sCur))
              ->addParam('purpose', $sPurpose)
              ->addParam('locale', $lang)
              ->addParam('tds2Address', $tds2Address)
              ->addParam('tds2Postcode', $tds2Postcode)
              ->addParam('tds2City', $tds2City)
              ->addParam('tds2Country', $tds2Country)
              ->addParam('tds2Optional', json_encode($aTdsOptionalInfo))
              ->addParam('urlRedirect', $urlRedirect)
              ->addParam('urlNotify', $urlNotify)
              ->addParam('sourceId', $sourceParam)
              ->addParam('orderId', $iOrderId)
              ->addParam('customerId', $iUserId)
              //the hash field is auto generated by the SDK
              ->submit();

            if (!$request->requestHasSucceeded()) {
              // change order status
              $oOrder->oxorder__oxtransstatus = new oxField('ERROR');
              $oOrder->save();
              oxRegistry::getSession()->setVariable('_girocheckout_payment_error', $request->getResponseMessage($request->getResponseParam('rc'), $lang));
              return parent::_getNextStep(oxOrder::ORDER_STATE_PAYMENTERROR);
            }
            else {
              $strUrlRedirect = $request->getResponseParam('redirect');
              oxRegistry::getUtils()->redirect($strUrlRedirect, false);
            }
          }
          catch (Exception $e) {
            $oOrder->oxorder__oxtransstatus = new oxField('ERROR');
            $oOrder->save();
            oxRegistry::getSession()->setVariable('_girocheckout_payment_error', GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $lang));
            return parent::_getNextStep(oxOrder::ORDER_STATE_PAYMENTERROR);
          }
        }

        // check that this is a directdebit payment
        if ($this->getPayment()->getId() == "gc_directdebit") {

          // change order status
          $oOrder->oxorder__oxtransstatus = new oxField('IN PAYMENT');
          $oOrder->oxorder__giroconnect__transaction_id = new oxField($transactionId);
          $oOrder->setPayment($oBasket->getPaymentId());
          $oOrder->save();

          try {
            $sPurpose = $this->getPurpose(oxRegistry::getConfig()->getConfigParam('GC_DIRECTDEBIT_PURPOSE'), $oOrder, $oUser);

            //Sends request to Girocheckout.
            $request = new GiroCheckout_SDK_Request('directDebitTransactionWithPaymentPage');
            $request->setSecret(oxRegistry::getConfig()->getConfigParam('GC_DIRECTDEBIT_SECRET'));
            $request->addParam('merchantId', oxRegistry::getConfig()->getConfigParam('GC_DIRECTDEBIT_MERCHANTID'))
              ->addParam('projectId', oxRegistry::getConfig()->getConfigParam('GC_DIRECTDEBIT_PROJECTID'))
              ->addParam('merchantTxId', $transactionId)
              ->addParam('amount', round($dAmount * 100))
              ->addParam('currency', strtoupper($sCur))
              ->addParam('purpose', $sPurpose)
              ->addParam('urlRedirect', $urlRedirect)
              ->addParam('urlNotify', $urlNotify)
              ->addParam('sourceId', $sourceParam)
              ->addParam('orderId', $iOrderId)
              ->addParam('customerId', $iUserId)
              ->submit();


            if ($request->requestHasSucceeded()) {
              $strUrlRedirect = $request->getResponseParam('redirect');
              oxRegistry::getUtils()->redirect($strUrlRedirect, false);
            }
            else {
              // change order status
              if (oxRegistry::getConfig()->getConfigParam('GC_DIRECTDEBIT_ORDER_SETTINGS') == 1) {
                $oOrder->delete();
              }
              else {

                $oOrder->oxorder__oxtransstatus = new oxField('ERROR');
                $oOrder->save();
                $oOrder->cancelOrder();
                oxRegistry::getSession()->setVariable('sess_challenge', oxUtilsObject::getInstance()->generateUID()); // <-- forces new order creation
                oxRegistry::getSession()->setVariable('_girocheckout_payment_error', $request->getResponseMessage($request->getResponseParam('rc'), $lang));
                return parent::_getNextStep(oxOrder::ORDER_STATE_PAYMENTERROR);
              }
            }
          }
          catch (Exception $e) {
            $oOrder->oxorder__oxtransstatus = new oxField('ERROR');
            $oOrder->save();
            $oOrder->cancelOrder();
            oxRegistry::getSession()->setVariable('sess_challenge', oxUtilsObject::getInstance()->generateUID()); // <-- forces new order creation
            oxRegistry::getSession()->setVariable('_girocheckout_payment_error', GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $lang));
            return parent::_getNextStep(oxOrder::ORDER_STATE_PAYMENTERROR);
          }
        }

        // check that this is a eps payment
        if ($this->getPayment()->getId() == "gc_eps") {

          // change order status
          $oOrder->oxorder__oxtransstatus = new oxField('IN PAYMENT');
          $oOrder->oxorder__giroconnect__transaction_id = new oxField($transactionId);
          $oOrder->setPayment($oBasket->getPaymentId());
          $oOrder->save();

          try {
            $sPurpose = $this->getPurpose(oxRegistry::getConfig()->getConfigParam('GC_EPS_PURPOSE'), $oOrder, $oUser);

            //Sends request to Girocheckout.
            $request = new GiroCheckout_SDK_Request('epsTransaction');
            $request->setSecret(oxRegistry::getConfig()->getConfigParam('GC_EPS_SECRET'));
            $request->addParam('merchantId', oxRegistry::getConfig()->getConfigParam('GC_EPS_MERCHANTID'))
              ->addParam('projectId', oxRegistry::getConfig()->getConfigParam('GC_EPS_PROJECTID'))
              ->addParam('merchantTxId', $transactionId)
              ->addParam('amount', round($dAmount * 100))
              ->addParam('currency', strtoupper($sCur))
              ->addParam('purpose', $sPurpose)
              ->addParam('bic', '')
              ->addParam('urlRedirect', $urlRedirect)
              ->addParam('urlNotify', $urlNotify)
              ->addParam('sourceId', $sourceParam)
              ->addParam('orderId', $iOrderId)
              ->addParam('customerId', $iUserId)
              ->submit();

            if (!$request->requestHasSucceeded()) {
// change order status
              $oOrder->oxorder__oxtransstatus = new oxField('ERROR');
              $oOrder->save();
              oxRegistry::getSession()->setVariable('_girocheckout_payment_error', $request->getResponseMessage($request->getResponseParam('rc'), $lang));
              return parent::_getNextStep(oxOrder::ORDER_STATE_PAYMENTERROR);
            }
            else {
              $strUrlRedirect = $request->getResponseParam('redirect');

//          oxUtils::getInstance()->redirect($strUrlRedirect);
              oxRegistry::getUtils()->redirect($strUrlRedirect, false);
            }
          }
          catch (Exception $e) {
            $oOrder->oxorder__oxtransstatus = new oxField('ERROR');
            $oOrder->save();
            oxRegistry::getSession()->setVariable('_girocheckout_payment_error', GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $lang));
            return parent::_getNextStep(oxOrder::ORDER_STATE_PAYMENTERROR);
          }
        }

        // check that this is a giropay payment
        if ($this->getPayment()->getId() == "gc_giropay") {

          // change order status
          $oOrder->oxorder__oxtransstatus = new oxField('IN PAYMENT');

          $oOrder->oxorder__giroconnect__transaction_id = new oxField($transactionId);
          $oOrder->setPayment($oBasket->getPaymentId());
          $oOrder->save();

          try {
            $sPurpose = $this->getGiropayPurpose(oxRegistry::getConfig()->getConfigParam('GC_GIROPAY_PURPOSE'), $oOrder, $oUser);
            $aShippingData = $this->getShippingData($oOrder);
            $iNumProdVirtual = 0;
            $iNumProdPhysical = 0;
            $strCartType = self::CART_TYPE_PHYSICAL;


            $oCart = new GiroCheckout_SDK_Request_Cart();
            $aBasketContents = $this->getBasket()->getContents();

            if (!empty($aBasketContents)) {
              foreach ($aBasketContents as $oBasketItem) {
                $sEAN = "";
                $oArticle = $oBasketItem->getArticle();
                $sEAN = $oArticle->oxarticles__oxean->value;
                $sNombre = strlen($oBasketItem->getTitle()) > 100 ? substr($oBasketItem->getTitle(), 0, 90) . '...' : $oBasketItem->getTitle();
                $iPrice = round($oBasketItem->getUnitPrice()->getBruttoPrice() * 100);
                $oCart->addItem($sNombre, $oBasketItem->getAmount(), $iPrice, $sEAN);

                if ($oArticle->oxarticles__oxisdownloadable->value == 1) {
                  $iNumProdVirtual++;
                } else {
                  $iNumProdPhysical++;
                }
              }
            }

            // Set the shopping cart type
            if ($iNumProdVirtual > 0 && $iNumProdPhysical == 0) {
              $strCartType = self::CART_TYPE_DIGITAL;

            }
            elseif ($iNumProdVirtual > 0 && $iNumProdPhysical > 0) {
              $strCartType = self::CART_TYPE_MIXED;
            }

            //Sends request to Girocheckout.
            $request = new GiroCheckout_SDK_Request('giropayTransaction');
            $request->setSecret(oxRegistry::getConfig()->getConfigParam('GC_GIROPAY_SECRET'));
            $request->addParam('merchantId', oxRegistry::getConfig()->getConfigParam('GC_GIROPAY_MERCHANTID'))
              ->addParam('projectId', oxRegistry::getConfig()->getConfigParam('GC_GIROPAY_PROJECTID'))
              ->addParam('merchantTxId', $transactionId)
              ->addParam('amount', round($dAmount * 100))
              ->addParam('currency', strtoupper($sCur))
              ->addParam('purpose', $sPurpose)
              ->addParam('bic', '')
              ->addParam('urlRedirect', $urlRedirect)
              ->addParam('urlNotify', $urlNotify)
              ->addParam('sourceId', $sourceParam)
              ->addParam('orderId', $iOrderId)
              ->addParam('customerId', $iUserId)
              ->addParam('shippingAddresseFirstName', $aShippingData["shippingAddresseFirstName"])
              ->addParam('shippingAddresseLastName', $aShippingData["shippingAddresseLastName"])
              ->addParam('shippingCompany', $aShippingData["shippingCompany"])
              ->addParam('shippingAdditionalAddressInformation', $aShippingData["shippingAdditionalAddressInformation"])
              ->addParam('shippingStreet', $aShippingData["shippingStreet"])
              ->addParam('shippingZipCode', $aShippingData["shippingZipCode"])
              ->addParam('shippingCity', $aShippingData["shippingCity"])
              ->addParam('shippingCountry', $aShippingData["shippingCountryIso"])
              ->addParam('shippingEmail',  $oOrder->oxorder__oxbillemail->value)
              ->addParam('cart', $oCart)
              ->addParam('shoppingCartType', $strCartType)

              //the hash field is auto generated by the SDK
              ->submit();

            if (!$request->requestHasSucceeded()) {
              // change order status
              $oOrder->oxorder__oxtransstatus = new oxField('ERROR');
              $oOrder->save();
              oxRegistry::getSession()->setVariable('_girocheckout_payment_error', $request->getResponseMessage($request->getResponseParam('rc'), $lang));
              return parent::_getNextStep(oxOrder::ORDER_STATE_PAYMENTERROR);
            }
            else {
              $strUrlRedirect = $request->getResponseParam('redirect');

              oxRegistry::getUtils()->redirect($strUrlRedirect, false);
            }
          }
          catch (Exception $e) {
            $oOrder->oxorder__oxtransstatus = new oxField('ERROR');
            $oOrder->save();
            oxRegistry::getSession()->setVariable('_girocheckout_payment_error', GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $lang));
            return parent::_getNextStep(oxOrder::ORDER_STATE_PAYMENTERROR);
          }
        }

        // check that this is a iDEAL payment
        if ($this->getPayment()->getId() == "gc_ideal") {

          // change order status
          $oOrder->oxorder__oxtransstatus = new oxField('IN PAYMENT');
          $oOrder->oxorder__giroconnect__transaction_id = new oxField($transactionId);
          $oOrder->setPayment($oBasket->getPaymentId());
          $oOrder->save();

          try {
            $sPurpose = $this->getPurpose(oxRegistry::getConfig()->getConfigParam('GC_IDEAL_PURPOSE'), $oOrder, $oUser);

            //Sends request to Girocheckout.
            $request = new GiroCheckout_SDK_Request('idealPayment');
            $request->setSecret(oxRegistry::getConfig()->getConfigParam('GC_IDEAL_SECRET'));
            $request->addParam('merchantId', oxRegistry::getConfig()->getConfigParam('GC_IDEAL_MERCHANTID'))
              ->addParam('projectId', oxRegistry::getConfig()->getConfigParam('GC_IDEAL_PROJECTID'))
              ->addParam('merchantTxId', $transactionId)
              ->addParam('amount', round($dAmount * 100))
              ->addParam('currency', strtoupper($sCur))
              ->addParam('purpose', $sPurpose)
              ->addParam('urlRedirect', $urlRedirect)
              ->addParam('urlNotify', $urlNotify)
              ->addParam('sourceId', $sourceParam)
              ->addParam('orderId', $iOrderId)
              ->addParam('customerId', $iUserId)
              ->submit();


            if (!$request->requestHasSucceeded()) {
              // change order status
              $oOrder->oxorder__oxtransstatus = new oxField('ERROR');
              $oOrder->save();
              oxRegistry::getSession()->setVariable('_girocheckout_payment_error', $request->getResponseMessage($request->getResponseParam('rc'), $lang));
              return parent::_getNextStep(oxOrder::ORDER_STATE_PAYMENTERROR);
            }
            else {
              $strUrlRedirect = $request->getResponseParam('redirect');
              oxRegistry::getUtils()->redirect($strUrlRedirect, false);
            }
          }
          catch (Exception $e) {
            $oOrder->oxorder__oxtransstatus = new oxField('ERROR');
            $oOrder->save();
            oxRegistry::getSession()->setVariable('_girocheckout_payment_error', GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $lang));
            return parent::_getNextStep(oxOrder::ORDER_STATE_PAYMENTERROR);
          }
        }

        // check that this is a paydirekt payment
        if ($this->getPayment()->getId() == "gc_paydirekt") {

          // change order status
          $oOrder->oxorder__oxtransstatus = new oxField('IN PAYMENT');
          $oOrder->oxorder__giroconnect__transaction_id = new oxField($transactionId);
          $oOrder->setPayment($oBasket->getPaymentId());
          $oOrder->save();

          try {
            $sPurpose = $this->getPurpose(oxRegistry::getConfig()->getConfigParam('GC_PAYDIREKT_PURPOSE'), $oOrder, $oUser);
            $aShippingData = $this->getShippingData($oOrder);
            $orderAmount = round(($dAmount - $aShippingData["shippingAmount"]) * 100);
            $oCart = new GiroCheckout_SDK_Request_Cart();
            $aBasketContents = $this->getBasket()->getContents();
            $oOrderNumber = $oOrder->oxorder__oxordernr->value;

            if (!empty($aBasketContents)) {
              foreach ($aBasketContents as $oBasketItem) {
                $sEAN = "";
                $oArticle = $oBasketItem->getArticle();
                $sEAN = $oArticle->oxarticles__oxean->value;
                $sNombre = strlen($oBasketItem->getTitle()) > 100 ? substr($oBasketItem->getTitle(), 0, 90) . '...' : $oBasketItem->getTitle();
                $iPrice = round($oBasketItem->getUnitPrice()->getBruttoPrice() * 100);
                $oCart->addItem($sNombre, $oBasketItem->getAmount(), $iPrice, $sEAN);
              }
            }

            //Sends request to Girocheckout.
            $request = new GiroCheckout_SDK_Request('paydirektTransaction');
            $request->setSecret(oxRegistry::getConfig()->getConfigParam('GC_PAYDIREKT_SECRET'));
            $request->addParam('merchantId', oxRegistry::getConfig()->getConfigParam('GC_PAYDIREKT_MERCHANTID'))
              ->addParam('projectId', oxRegistry::getConfig()->getConfigParam('GC_PAYDIREKT_PROJECTID'))
              ->addParam('merchantTxId', $transactionId)
              ->addParam('amount', round($dAmount * 100))
              ->addParam('currency', strtoupper($sCur))
              ->addParam('purpose', $sPurpose)
              ->addParam('shippingAmount', round($aShippingData["shippingAmount"] * 100))
              ->addParam('shippingAddresseFirstName', $aShippingData["shippingAddresseFirstName"])
              ->addParam('shippingAddresseLastName', $aShippingData["shippingAddresseLastName"])
              ->addParam('shippingCompany', $aShippingData["shippingCompany"])
              ->addParam('shippingAdditionalAddressInformation', $aShippingData["shippingAdditionalAddressInformation"])
              ->addParam('shippingStreet', $aShippingData["shippingStreet"])
              ->addParam('shippingZipCode', $aShippingData["shippingZipCode"])
              ->addParam('shippingCity', $aShippingData["shippingCity"])
              ->addParam('shippingCountry', $aShippingData["shippingCountryIso"])
              ->addParam('orderAmount', $orderAmount)
              ->addParam('cart', $oCart)
              ->addParam('urlRedirect', $urlRedirect)
              ->addParam('urlNotify', $urlNotify)
              ->addParam('sourceId', $sourceParam)
              ->addParam('orderId', $oOrderNumber)
              ->addParam('customerId', $iUserNr)
              ->addParam('customerMail', $oOrder->oxorder__oxbillemail->value)
              //the hash field is auto generated by the SDK
              ->submit();

            if (!$request->requestHasSucceeded()) {
              // change order status
              $oOrder->oxorder__oxtransstatus = new oxField('ERROR');
              $oOrder->save();
              oxRegistry::getSession()->setVariable('_girocheckout_payment_error', $request->getResponseMessage($request->getResponseParam('rc'), $lang));
              return parent::_getNextStep(oxOrder::ORDER_STATE_PAYMENTERROR);
            }
            else {
              $strUrlRedirect = $request->getResponseParam('redirect');
              oxRegistry::getUtils()->redirect($strUrlRedirect, false);
            }
          }
          catch (Exception $e) {
            $oOrder->oxorder__oxtransstatus = new oxField('ERROR');
            $oOrder->save();
            oxRegistry::getSession()->setVariable('_girocheckout_payment_error', GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $lang));
            return parent::_getNextStep(oxOrder::ORDER_STATE_PAYMENTERROR);
          }
        }

        // check that this is a SOFORT Überweisung payment
        if ($this->getPayment()->getId() == "gc_sofortuw") {

          // change order status
          $oOrder->oxorder__oxtransstatus = new oxField('IN PAYMENT');
          $oOrder->oxorder__giroconnect__transaction_id = new oxField($transactionId);
          $oOrder->setPayment($oBasket->getPaymentId());
          $oOrder->save();

          try {
            $sPurpose = $this->getPurpose(oxRegistry::getConfig()->getConfigParam('GC_SOFORTUW_PURPOSE'), $oOrder, $oUser);

            //Sends request to Girocheckout.
            $request = new GiroCheckout_SDK_Request('sofortuwTransaction');
            $request->setSecret(oxRegistry::getConfig()->getConfigParam('GC_SOFORTUW_SECRET'));
            $request->addParam('merchantId', oxRegistry::getConfig()->getConfigParam('GC_SOFORTUW_MERCHANTID'))
              ->addParam('projectId', oxRegistry::getConfig()->getConfigParam('GC_SOFORTUW_PROJECTID'))
              ->addParam('merchantTxId', $transactionId)
              ->addParam('amount', round($dAmount * 100))
              ->addParam('currency', strtoupper($sCur))
              ->addParam('purpose', $sPurpose)
              ->addParam('urlRedirect', $urlRedirect)
              ->addParam('urlNotify', $urlNotify)
              ->addParam('sourceId', $sourceParam)
              ->addParam('orderId', $iOrderId)
              ->addParam('customerId', $iUserId)
              ->submit();

            if (!$request->requestHasSucceeded()) {
              // change order status
              $oOrder->oxorder__oxtransstatus = new oxField('ERROR');
              $oOrder->save();
              oxRegistry::getSession()->setVariable('_girocheckout_payment_error', $request->getResponseMessage($request->getResponseParam('rc'), $lang));
              return parent::_getNextStep(oxOrder::ORDER_STATE_PAYMENTERROR);
            }
            else {
              $strUrlRedirect = $request->getResponseParam('redirect');

              oxRegistry::getUtils()->redirect($strUrlRedirect, false);
            }
          }
          catch (Exception $e) {
            $oOrder->oxorder__oxtransstatus = new oxField('ERROR');
            $oOrder->save();
            oxRegistry::getSession()->setVariable('_girocheckout_payment_error', GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $lang));
            return parent::_getNextStep(oxOrder::ORDER_STATE_PAYMENTERROR);
          }
        }
        // check that this is a Maestro payment
        if ($this->getPayment()->getId() == "gc_maestro") {

          // change order status
          $oOrder->oxorder__oxtransstatus = new oxField('IN PAYMENT');
          $oOrder->oxorder__giroconnect__transaction_id = new oxField($transactionId);
          $oOrder->setPayment($oBasket->getPaymentId());
          $oOrder->save();

          try {
            $sPurpose = $this->getPurpose(oxRegistry::getConfig()->getConfigParam('GC_MAESTRO_PURPOSE'), $oOrder, $oUser);

            //Sends request to Girocheckout.
            $request = new GiroCheckout_SDK_Request('maestroTransaction');
            $request->setSecret(oxRegistry::getConfig()->getConfigParam('GC_MAESTRO_SECRET'));
            $request->addParam('merchantId', oxRegistry::getConfig()->getConfigParam('GC_MAESTRO_MERCHANTID'))
              ->addParam('projectId', oxRegistry::getConfig()->getConfigParam('GC_MAESTRO_PROJECTID'))
              ->addParam('merchantTxId', $transactionId)
              ->addParam('amount', round($dAmount * 100))
              ->addParam('currency', strtoupper($sCur))
              ->addParam('purpose', $sPurpose)
              ->addParam('urlRedirect', $urlRedirect)
              ->addParam('urlNotify', $urlNotify)
              ->addParam('sourceId', $sourceParam)
              ->addParam('orderId', $iOrderId)
              ->addParam('customerId', $iUserId)
              ->submit();

            if (!$request->requestHasSucceeded()) {
              // change order status
              $oOrder->oxorder__oxtransstatus = new oxField('ERROR');
              $oOrder->save();
              oxRegistry::getSession()->setVariable('_girocheckout_payment_error', $request->getResponseMessage($request->getResponseParam('rc'), $lang));
              return parent::_getNextStep(oxOrder::ORDER_STATE_PAYMENTERROR);
            }
            else {
              $strUrlRedirect = $request->getResponseParam('redirect');

              oxRegistry::getUtils()->redirect($strUrlRedirect, false);
            }
          }
          catch (Exception $e) {
            $oOrder->oxorder__oxtransstatus = new oxField('ERROR');
            $oOrder->save();
            oxRegistry::getSession()->setVariable('_girocheckout_payment_error', GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $lang));
            return parent::_getNextStep(oxOrder::ORDER_STATE_PAYMENTERROR);
          }
        }

        // check that this is a Bluecode payment
        if ($this->getPayment()->getId() == "gc_bluecode") {

          // change order status
          $oOrder->oxorder__oxtransstatus = new oxField('IN PAYMENT');
          $oOrder->oxorder__giroconnect__transaction_id = new oxField($transactionId);
          $oOrder->setPayment($oBasket->getPaymentId());
          $oOrder->save();

          try {
            $sPurpose = $this->getPurpose(oxRegistry::getConfig()->getConfigParam('GC_BLUECODE_PURPOSE'), $oOrder, $oUser);

            //Sends request to Girocheckout.
            $request = new GiroCheckout_SDK_Request('blueCodeTransaction');
            $request->setSecret(oxRegistry::getConfig()->getConfigParam('GC_BLUECODE_SECRET'));
            $request->addParam('merchantId', oxRegistry::getConfig()->getConfigParam('GC_BLUECODE_MERCHANTID'))
              ->addParam('projectId', oxRegistry::getConfig()->getConfigParam('GC_BLUECODE_PROJECTID'))
              ->addParam('merchantTxId', $transactionId)
              ->addParam('amount', round($dAmount * 100))
              ->addParam('currency', strtoupper($sCur))
              ->addParam('purpose', $sPurpose)
              ->addParam('urlRedirect', $urlRedirect)
              ->addParam('urlNotify', $urlNotify)
              ->addParam('sourceId', $sourceParam)
              ->addParam('orderId', $iOrderId)
              ->addParam('customerId', $iUserId)
              ->submit();

            if (!$request->requestHasSucceeded()) {
              // change order status
              $oOrder->oxorder__oxtransstatus = new oxField('ERROR');
              $oOrder->save();
              oxRegistry::getSession()->setVariable('_girocheckout_payment_error', $request->getResponseMessage($request->getResponseParam('rc'), $lang));
              return parent::_getNextStep(oxOrder::ORDER_STATE_PAYMENTERROR);
            }
            else {
              $strUrlRedirect = $request->getResponseParam('redirect');

              oxRegistry::getUtils()->redirect($strUrlRedirect, false);
            }
          }
          catch (Exception $e) {
            $oOrder->oxorder__oxtransstatus = new oxField('ERROR');
            $oOrder->save();
            oxRegistry::getSession()->setVariable('_girocheckout_payment_error', GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $lang));
            return parent::_getNextStep(oxOrder::ORDER_STATE_PAYMENTERROR);
          }
        }
      }
    }
    return parent::_getNextStep($iSuccess);
  }

  /**
   * - order nr reservieren
   * - order löschen
   * - order objekt erstellen
   * - reservierte order nr übernehmen
   * - order finalisieren ($oOrder->finalizeOrder($this->getBasket(), $this->getUser()))
   * - onOrderExecute ($oUser->onOrderExecute($oBasket, $iSuccess))
   *
   * @param oxOrder $oOrder
   *
   * @return array
   */
  protected function girocheckout__recreateOrder(oxOrder $oOrder) {
    $oOrderNumber = $oOrder->oxorder__oxordernr->value;

// create order number reservation
    /** @var girocheckout__order_number_reservation $oOrderNumberReservation */
    $oOrderNumberReservation = oxNew('girocheckout__order_number_reservation');
    $reservationKey = girocheckout__order_number_reservation::getReservationKey($oOrderNumber);
    if (!$oOrderNumberReservation->load($reservationKey)) {
      $oOrderNumberReservation->setId($reservationKey);
      $oOrderNumberReservation->save();
    }

    $oOrder->delete();
    /** @var oxorder $newOrder */
    $newOrder = oxNew('oxorder');
    $newOrder->oxorder__oxordernr = new oxField($oOrderNumber, oxField::T_RAW);
    $iSuccess = $newOrder->finalizeOrder($this->getBasket(), $this->getUser());
    $this->getUser()->onOrderExecute($this->getBasket(), $iSuccess);

    // delete order number reservation
    $oOrderNumberReservation->delete();

    return array($newOrder, $iSuccess);
  }

  /**
   * handles GiroCheckout notify action.
   *
   * @author S-Public Services GmbH
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, S-Public Services GmbH
   *
   */
  public function processGiroCheckoutNotify() {
    if (empty($_GET['pm'])) {
      header('HTTP/1.1 400 Bad Request');
      echo "param pm not found.";
      exit;
    }

    $pm = $_GET['pm'];
    $lang = "en";

    $sess_challenge = $_GET['sess_challenge'];

    /** @var girocheckout__oxorder $oOrder */
    $oOrder = oxNew('oxorder');
    $oOrder->load($sess_challenge);

    if (!$oOrder->isLoaded()) {
      header('HTTP/1.1 400 Bad Request');
      echo "Order not found";
      exit;
    }
    $iUserId = $oOrder->oxorder__oxuserid->value;
    $iOrderId = $oOrder->oxorder__oxid->value;

    try {
      $notify = null;

      //Get the notification
      switch ($pm) {
        case "gc_creditcard":
          $notify = new GiroCheckout_SDK_Notify('creditCardTransaction');
          $notify->setSecret(oxRegistry::getConfig()->getConfigParam('GC_CREDITCARD_SECRET'));
          break;

        case "gc_directdebit":
          $notify = new GiroCheckout_SDK_Notify('directDebitTransactionWithPaymentPage');
          $notify->setSecret(oxRegistry::getConfig()->getConfigParam('GC_DIRECTDEBIT_SECRET'));

          break;

        case "gc_giropay":
          $notify = new GiroCheckout_SDK_Notify('giropayTransaction');
          $notify->setSecret(oxRegistry::getConfig()->getConfigParam('GC_GIROPAY_SECRET'));
          break;

        case "gc_ideal":
          $notify = new GiroCheckout_SDK_Notify('idealPayment');
          $notify->setSecret(oxRegistry::getConfig()->getConfigParam('GC_IDEAL_SECRET'));
          break;

        case "gc_eps":
          $notify = new GiroCheckout_SDK_Notify('epsTransaction');
          $notify->setSecret(oxRegistry::getConfig()->getConfigParam('GC_EPS_SECRET'));
          break;

        case "gc_paydirekt":
          $notify = new GiroCheckout_SDK_Notify('paydirektTransaction');
          $notify->setSecret(oxRegistry::getConfig()->getConfigParam('GC_PAYDIREKT_SECRET'));
          break;

        case "gc_sofortuw":
          $notify = new GiroCheckout_SDK_Notify('sofortuwTransaction');
          $notify->setSecret(oxRegistry::getConfig()->getConfigParam('GC_SOFORTUW_SECRET'));
          break;

        case "gc_maestro":
          $notify = new GiroCheckout_SDK_Notify('maestroTransaction');
          $notify->setSecret(oxRegistry::getConfig()->getConfigParam('GC_MAESTRO_SECRET'));
          break;

        case "gc_bluecode":
          $notify = new GiroCheckout_SDK_Notify('blueCodeTransaction');
          $notify->setSecret(oxRegistry::getConfig()->getConfigParam('GC_BLUECODE_SECRET'));
          break;
      }

      $notify->parseNotification($_GET);

      if ($oOrder->oxorder__giroconnect__status != "" && $oOrder->oxorder__oxtransstatus != "IN PAYMENT") {
        if ($oOrder->oxorder__oxtransstatus == "OK") {
          $strReqStatus = "ok";
          $strResult = "OK";
          $strErrorMsg = "";
        }
        elseif ($oOrder->oxorder__oxtransstatus == "ERROR") {
          $strReqStatus = "ok";
          $strResult = "OK";
          $strErrorMsg = $notify->getResponseMessage($notify->getResponseParam('gcResultPayment'), $lang);
        }
        else {
          $strReqStatus = "ok";
          $strResult = "OK";
          $strErrorMsg = $notify->getResponseMessage($notify->getResponseParam('gcResultPayment'), $lang);
        }
      }
      elseif ($oOrder->oxorder__giroconnect__transaction_id != $_GET['gcMerchantTxId']) {
        $strReqStatus = "bad";
        $strResult = "ERROR";
        $strErrorMsg = "Order tx id does not match merchantTxId";
      }
      elseif ($oOrder->oxorder__oxpaymenttype->value != $pm) {
        $strReqStatus = "bad";
        $strResult = "ERROR";
        $strErrorMsg = "Parameter pm not valid for order";
      }
      elseif (!$notify->paymentSuccessful() || $oOrder->oxorder__oxtransstatus == "ERROR") {
        // TRANSACTION FAILED

        // Is order set to delete on failure?
        if ((oxRegistry::getConfig()->getConfigParam('GC_GIROPAY_ORDER_SETTINGS') == 1 && $pm == "gc_giropay") ||
          (oxRegistry::getConfig()->getConfigParam('GC_CREDITCARD_ORDER_SETTINGS') == 1 && $pm == "gc_credictard") ||
          (oxRegistry::getConfig()->getConfigParam('GC_DIRECTDEBIT_ORDER_SETTINGS') == 1 && $pm == "gc_directdebit") ||
          (oxRegistry::getConfig()->getConfigParam('GC_IDEAL_ORDER_SETTINGS') == 1 && $pm == "gc_ideal") ||
          (oxRegistry::getConfig()->getConfigParam('GC_EPS_ORDER_SETTINGS') == 1 && $pm == "gc_eps") ||
          (oxRegistry::getConfig()->getConfigParam('GC_PAYDIREKT_ORDER_SETTINGS') == 1 && $pm == "gc_paydirekt") ||
          (oxRegistry::getConfig()->getConfigParam('GC_SOFORTUW_ORDER_SETTINGS') == 1 && $pm == "gc_sofortuw") ||
          (oxRegistry::getConfig()->getConfigParam('GC_MAESTRO_ORDER_SETTINGS') == 1 && $pm == "gc_maestro") ||
          (oxRegistry::getConfig()->getConfigParam('GC_BLUECODE_ORDER_SETTINGS') == 1 && $pm == "gc_bluecode")
        ) {

          $oOrder->delete();
        }
        else {
          $oOrder->oxorder__oxtransstatus = new oxField('ERROR');
          $oOrder->oxorder__giroconnect__status = new oxField($notify->getResponseParam('gcResultPayment'));
          $oOrder->save();
          $oOrder->cancelOrder();
        }

        $strReqStatus = "ok";
        $strResult = "OK";
        $strErrorMsg = $notify->getResponseMessage($notify->getResponseParam('gcResultPayment'), $lang);
      }
      else {
        // TRANSACTION SUCCESS

        $oOrder->oxorder__oxtransstatus = new oxField('OK');
        $oOrder->oxorder__giroconnect__status = new oxField($notify->getResponseParam('gcResultPayment'));
        $oOrder->oxorder__oxpaid = new oxField(oxRegistry::get("oxUtilsDate")->formatDBDate(date("Y-m-d H:i:s"), true));
        $oOrder->save();
        oxRegistry::getSession()->setVariable('girocheckout_disable_article_check', '1');
        $oOrder->sendGiroCheckoutOrderByEmail();
        oxRegistry::getSession()->deleteVariable('girocheckout_disable_article_check');

        $strReqStatus = "ok";
        $strResult = "OK";
        $strErrorMsg = "";
      }
    }
    catch (Exception $e) {
      $strReqStatus = "bad";
      $strResult = "ERROR";
      $strErrorMsg = "Exception in notify: " . $e->getMessage();
    }

    switch ($strReqStatus) {
      case "bad":
        $notify->sendBadRequestStatus();
        break;
      default:
        $notify->sendOkStatus();
        break;
    }

    $notify->setNotifyResponseParam('Result', $strResult);
    $notify->setNotifyResponseParam('ErrorMessage', $strErrorMsg);
    $notify->setNotifyResponseParam('MailSent', '');
    $notify->setNotifyResponseParam('OrderId', $iOrderId);
    $notify->setNotifyResponseParam('CustomerId', $iUserId);
    echo $notify->getNotifyResponseStringJson();
    header("HTTP/1.1 200 OK");
    exit;
  }

  /**
   * handles GiroCheckout redirect action.
   *
   * @author S-Public Services GmbH
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, S-Public Services GmbH
   *
   */
  public function processGiroCheckoutRedirect() {
    $pm = $_GET['pm'];

    if (empty($_GET['pm'])) {
      exit;
    }
    $lang = strtoupper(oxRegistry::getLang()->getLanguageAbbr());

    $sess_challenge = $_GET['sess_challenge'];

    /** @var girocheckout__oxorder $oOrder */
    $oOrder = oxNew('oxorder');
    $oOrder->load($sess_challenge);

    $sErrorMsg = "";

    try {
      $notify = null;

      //Get the notification
      switch ($pm) {
        case "gc_creditcard":
          $notify = new GiroCheckout_SDK_Notify('creditCardTransaction');
          $notify->setSecret(oxRegistry::getConfig()->getConfigParam('GC_CREDITCARD_SECRET'));
          break;

        case "gc_directdebit":
          $notify = new GiroCheckout_SDK_Notify('directDebitTransactionWithPaymentPage');
          $notify->setSecret(oxRegistry::getConfig()->getConfigParam('GC_DIRECTDEBIT_SECRET'));
          break;

        case "gc_giropay":
          $notify = new GiroCheckout_SDK_Notify('giropayTransaction');
          $notify->setSecret(oxRegistry::getConfig()->getConfigParam('GC_GIROPAY_SECRET'));
          break;

        case "gc_ideal":
          $notify = new GiroCheckout_SDK_Notify('idealPayment');
          $notify->setSecret(oxRegistry::getConfig()->getConfigParam('GC_IDEAL_SECRET'));
          break;

        case "gc_eps":
          $notify = new GiroCheckout_SDK_Notify('epsTransaction');
          $notify->setSecret(oxRegistry::getConfig()->getConfigParam('GC_EPS_SECRET'));
          break;

        case "gc_paydirekt":
          $notify = new GiroCheckout_SDK_Notify('paydirektTransaction');
          $notify->setSecret(oxRegistry::getConfig()->getConfigParam('GC_PAYDIREKT_SECRET'));
          break;

        case "gc_sofortuw":
          $notify = new GiroCheckout_SDK_Notify('sofortuwTransaction');
          $notify->setSecret(oxRegistry::getConfig()->getConfigParam('GC_SOFORTUW_SECRET'));
          break;

        case "gc_maestro":
          $notify = new GiroCheckout_SDK_Notify('maestroTransaction');
          $notify->setSecret(oxRegistry::getConfig()->getConfigParam('GC_MAESTRO_SECRET'));
          break;

        case "gc_bluecode":
          $notify = new GiroCheckout_SDK_Notify('blueCodeTransaction');
          $notify->setSecret(oxRegistry::getConfig()->getConfigParam('GC_BLUECODE_SECRET'));
          break;
      }
      $notify->parseNotification($_GET);

      if ($oOrder->oxorder__giroconnect__status != "" && $oOrder->oxorder__oxtransstatus != "IN PAYMENT") {
        if ($oOrder->oxorder__oxtransstatus == "OK") {
          $strResult = "OK";
          $sErrorMsg = "";
        }
        elseif ($oOrder->oxorder__oxtransstatus == "ERROR") {
          $strResult = "ERROR";
          $sErrorMsg = $notify->getResponseMessage($notify->getResponseParam('gcResultPayment'), $lang);
        }
        else {
          $strResult = "ERROR";
          $sErrorMsg = $notify->getResponseMessage($notify->getResponseParam('gcResultPayment'), $lang);
        }
      }
      elseif ($oOrder->oxorder__giroconnect__transaction_id != $_GET['gcMerchantTxId']) {
        $strResult = "ERROR";
        $sErrorMsg = "Order tx id does not match merchantTxId";
      }
      elseif ($oOrder->oxorder__oxpaymenttype->value != $pm) {
        $strResult = "ERROR";
        $sErrorMsg = "Parameter pm not valid for order";
      }
      elseif (!$notify->paymentSuccessful()) {
        //TRANSACTION FAILED

        // Is order set to delete on failure?
        if ($oOrder->isLoaded()) {
          if ((oxRegistry::getConfig()->getConfigParam('GC_GIROPAY_ORDER_SETTINGS') == 1 && $pm == "gc_giropay") ||
            (oxRegistry::getConfig()->getConfigParam('GC_CREDITCARD_ORDER_SETTINGS') == 1 && $pm == "gc_credictard") ||
            (oxRegistry::getConfig()->getConfigParam('GC_DIRECTDEBIT_ORDER_SETTINGS') == 1 && $pm == "gc_directdebit") ||
            (oxRegistry::getConfig()->getConfigParam('GC_IDEAL_ORDER_SETTINGS') == 1 && $pm == "gc_ideal") ||
            (oxRegistry::getConfig()->getConfigParam('GC_EPS_ORDER_SETTINGS') == 1 && $pm == "gc_eps") ||
            (oxRegistry::getConfig()->getConfigParam('GC_PAYDIREKT_ORDER_SETTINGS') == 1 && $pm == "gc_paydirekt") ||
            (oxRegistry::getConfig()->getConfigParam('GC_SOFORTUW_ORDER_SETTINGS') == 1 && $pm == "gc_sofortuw") ||
            (oxRegistry::getConfig()->getConfigParam('GC_MAESTRO_ORDER_SETTINGS') == 1 && $pm == "gc_maestro") ||
            (oxRegistry::getConfig()->getConfigParam('GC_BLUECODE_ORDER_SETTINGS') == 1 && $pm == "gc_bluecode")
          ) {
            $oOrder->delete();
          }
          else {
            $oOrder->oxorder__oxtransstatus = new oxField('ERROR');
            $oOrder->oxorder__giroconnect__status = new oxField($notify->getResponseParam('gcResultPayment'));
            $oOrder->save();
            $oOrder->cancelOrder();
          }
        }
        $sErrorMsg = $notify->getResponseMessage($notify->getResponseParam('gcResultPayment'), $lang);

        //AGE VERIFICATION SERVICE
        if ($pm == "gc_giropay") {
          $iReturnCodeAVS = $notify->getResponseParam('gcResultAVS');
          if (!$notify->avsSuccessful() && $iReturnCodeAVS != "4900" && ($iReturnCodeAVS == "4020" || $iReturnCodeAVS == "4021" || $iReturnCodeAVS == "4022")) {
            $paymentAVSMsg = $notify->getResponseMessage($iReturnCodeAVS, $lang);
            if (!empty($paymentAVSMsg)) {
              $sErrorMsg .= " (" . $paymentAVSMsg . ")";
            }
          }
        }

        $strResult = "ERROR";
      }
      else {
        //TRANSACTION SUCCESS
        $strResult = "OK";
        $sErrorMsg = "";
      }

      switch ($strResult) {
        case "ERROR":
          oxRegistry::getSession()->setVariable(
            'sess_challenge', oxUtilsObject::getInstance()->generateUID()
          ); // <-- forces new order creation
          oxRegistry::getSession()->setVariable('_girocheckout_payment_error', $sErrorMsg);
          oxRegistry::getUtils()->redirect($this->getConfig()->getSslShopUrl() . 'index.php?cl=payment');
          break;
        default:
          oxRegistry::getSession()->deleteVariable('girocheckout_ideal_issuers');
          oxRegistry::getUtils()->redirect($this->getConfig()->getSslShopUrl() . 'index.php?cl=thankyou');
          break;
      }
    }
    catch (Exception $e) {
      oxRegistry::getSession()->setVariable(
        'sess_challenge', oxUtilsObject::getInstance()->generateUID()
      ); // <-- forces new order creation
      oxRegistry::getSession()->setVariable('_girocheckout_payment_error', GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $lang));
      oxRegistry::getUtils()->redirect($this->getConfig()->getSslShopUrl() . 'index.php?cl=payment');
    }
  }

  /**
   * Get the payment source id parameter.
   *
   * @author S-Public Services GmbH
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, S-Public Services GmbH
   * @return int
   */
  public function getGcSource() {
    /** @var oxModule $modul */
    $modul = oxNew('oxModule');
    $modul->load('girocheckout');

    return "Oxid " . oxRegistry::getConfig()->getVersion() . ";Oxid Plugin " . $modul->getInfo('version');
  }

  /**
   * Get the payment purpose.
   *
   * @author S-Public Services GmbH
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, S-Public Services GmbH
   * @return int
   */
  public function getPurpose($purpose, $oOrder, $oUser) {
    $strPurpose = $purpose;

    try {

      $oShop = oxNew("oxshop");
      $oShop->load($this->getConfig()->getShopId());

      $iCustomerNr = $oUser->oxuser__oxcustnr->value;
      $strLastName = $oOrder->oxorder__oxbilllname->value;
      $strFirstName = $oOrder->oxorder__oxbillfname->value;
      $strName = $strFirstName . ", " . $strLastName;

      $iOrderId = $oOrder->oxorder__oxordernr->value;

      $strShopName = $oShop->oxshops__oxname->getRawValue();

      if (empty($strPurpose)) {
        $strPurpose = "{ORDERID}, {SHOPNAME}";
      }

      $strPurpose = str_replace("{ORDERID}", $iOrderId, $strPurpose);
      $strPurpose = str_replace("{CUSTOMERID}", $iCustomerNr, $strPurpose);
      $strPurpose = str_replace("{SHOPNAME}", $strShopName, $strPurpose);
      $strPurpose = str_replace("{CUSTOMERNAME}", $strName, $strPurpose);
      $strPurpose = str_replace("{CUSTOMERFIRSTNAME}", $strFirstName, $strPurpose);
      $strPurpose = str_replace("{CUSTOMERLASTNAME}", $strLastName, $strPurpose);
    }
    catch (Exception $e) {
      throw new Exception("Exception in getPurpose(): " . $e->getMessage());
    }

    if (empty($strPurpose)) {
      if (!empty($strFirstName) || !empty($strLastName)) {
        $strPurpose = "Kunde: " . $strFirstName . " " . $strLastName;
      }
      else {
        if (!empty($strShopName)) {
          $strPurpose = "Bestellung: " . $strShopName;
        }
        else {
          $strPurpose = "Bestellung: Oxid";
        }
      }
    }

    return substr($strPurpose, 0, 27);
  }

  /**
   * Get shipping information for paydirekt.
   *
   * @author S-Public Services GmbH
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, S-Public Services GmbH
   * @return array
   *
   */
  public function getShippingData($oOrder) {

    $aReturn = [];
    try {

      if (!empty($oOrder->oxorder__oxdellname->value)) {
        $aReturn["shippingAddresseFirstName"] = $oOrder->oxorder__oxdelfname->value;
        $aReturn["shippingAddresseLastName"] = $oOrder->oxorder__oxdellname->value;
        $aReturn["shippingCompany"] = $oOrder->oxorder__oxdelcompany->value;
        $aReturn["shippingAdditionalAddressInformation"] = $oOrder->oxorder__oxdeladdinfo->value;

        $aReturn["shippingStreet"] = $oOrder->oxorder__oxdelstreet->value . " " . $oOrder->oxorder__oxdelstreetnr->value;
        $aReturn["shippingZipCode"] = $oOrder->oxorder__oxdelzip->value;
        $aReturn["shippingCity"] = $oOrder->oxorder__oxdelcity->value;

        $oCountry = oxNew("oxcountry");
        $oCountry->load($oOrder->oxorder__oxdelcountryid->value);

        $aReturn["shippingCountryIso"] = $oCountry->oxcountry__oxisoalpha2->value;
      }
      else {
        if ($oOrder->oxorder__oxbilllname->value) {
          $aReturn["shippingAddresseFirstName"] = $oOrder->oxorder__oxbillfname->value;
          $aReturn["shippingAddresseLastName"] = $oOrder->oxorder__oxbilllname->value;
          $aReturn["shippingCompany"] = $oOrder->oxorder__oxbillcompany->value;
          $aReturn["shippingAdditionalAddressInformation"] = $oOrder->oxorder__oxbilladdinfo->value;

          $aReturn["shippingStreet"] = $oOrder->oxorder__oxbillstreet->value . " " . $oOrder->oxorder__oxbillstreetnr->value;
          $aReturn["shippingZipCode"] = $oOrder->oxorder__oxbillzip->value;
          $aReturn["shippingCity"] = $oOrder->oxorder__oxbillcity->value;

          $oCountry = oxNew("oxcountry");
          $oCountry->load($oOrder->oxorder__oxbillcountryid->value);

          $aReturn["shippingCountryIso"] = $oCountry->oxcountry__oxisoalpha2->value;
        }
      }

      $aReturn["shippingAmount"] = $oOrder->getOrderDeliveryPrice()->getBruttoPrice();
    }
    catch (Exception $exception) {
      throw new Exception("Exception in getShippingData(): " . $exception->getMessage());
    }

    return $aReturn;
  }

  /**
   * Get the payment purpose for giropay payment.
   *
   * @author S-Public Services GmbH
   * @package GiroCheckout
   * @copyright Copyright (c) 2022, S-Public Services GmbH
   * @return string
   */
  public function getGiropayPurpose($purpose, $oOrder, $oUser) {
    $strPurpose = $purpose;

    try {
      $oShop = oxNew("oxshop");
      $oShop->load($this->getConfig()->getShopId());

      $iCustomerNr = $oUser->oxuser__oxcustnr->value;
      $strLastName = $oOrder->oxorder__oxbilllname->value;
      $strFirstName = $oOrder->oxorder__oxbillfname->value;
      $strName = $strFirstName . ", " . $strLastName;

      $iOrderId = $oOrder->oxorder__oxordernr->value;

      $strShopName = $oShop->oxshops__oxname->getRawValue();

      if (empty($strPurpose)) {
        $strPurpose = "{ORDERID}, {SHOPNAME}";
      }
    }
    catch (Exception $e) {
      throw new Exception("Exception in getPurpose(): " . $e->getMessage());
    }

    $strPurpose = str_replace("{ORDERID}", $iOrderId, $strPurpose);
    $strPurpose = str_replace("{CUSTOMERID}", $iCustomerNr, $strPurpose);
    $strPurpose = str_replace("{SHOPNAME}", $strShopName, $strPurpose);
    $strPurpose = str_replace("{CUSTOMERNAME}", $strName, $strPurpose);
    $strPurpose = str_replace("{CUSTOMERFIRSTNAME}", $strFirstName, $strPurpose);
    $strPurpose = str_replace("{CUSTOMERLASTNAME}", $strLastName, $strPurpose);

    $bValidPurpose = self::validateGiropayPurpose($strPurpose);

    if (!$bValidPurpose) {
      $strPurposeFormat = preg_replace("([^[A-Za-z0-9\+\?\-\:\(\)\.\,\'\/])", '', $strPurpose);
    }
    else {
      $strPurposeFormat = $strPurpose;
    }

    $strPurposeFormat = substr($strPurposeFormat, 0, 27);

    return $strPurposeFormat;
  }

  /**
   * Validate the payment purpose for giropay payment.
   *
   * @author S-Public Services GmbH
   * @package GiroCheckout
   * @copyright Copyright (c) 2022, S-Public Services GmbH
   * @return array
   */
  public function validateGiropayPurpose($purpose) {
    $strPurpose = $purpose;

    if (!empty($strPurpose)) {
      if (preg_match("/^[A-Za-z0-9\+\?\-\:\(\)\.\,\'\/\s]+$/", $strPurpose)) {
        return true;
      }
      else {
        return false;
      }
    }

    return true;
  }
}
