<?php

require_once getShopBasePath() . 'modules/girocheckout/lib/GiroCheckout_SDK/GiroCheckout_SDK.php';

/**
 * Class girocheckout__payment
 */
class girocheckout__payment extends girocheckout__payment_parent {

    public $_girocheckout_payment_error;

    /**
     * return error message to display.
     *
     * @author S-Public Services GmbH
     * @package GiroCheckout
     * @copyright Copyright (c) 2016, S-Public Services GmbH
     * @return string
     * 
     */
    public function getGirocheckoutPaymentError() {
        if (oxRegistry::getSession()->hasVariable('_girocheckout_payment_error')) {
            $this->_girocheckout_payment_error = oxRegistry::getSession()->getVariable('_girocheckout_payment_error');
            oxRegistry::getSession()->deleteVariable('_girocheckout_payment_error');
        }

        return $this->_girocheckout_payment_error;
    }

    /**
     * check if the error message is from girocheckout.
     *
     * @author S-Public Services GmbH
     * @package GiroCheckout
     * @copyright Copyright (c) 2016, S-Public Services GmbH
     * @return boolean
     * 
     */
    public function isGirocheckoutPaymentError() {
        if (!empty($this->_girocheckout_payment_error) || oxRegistry::getSession()->hasVariable('_girocheckout_payment_error')) {
            return true;
        }

        return false;
    }

    /**
     * validate payment input data from customer.
     *
     * @author S-Public Services GmbH
     * @package GiroCheckout
     * @copyright Copyright (c) 2016, S-Public Services GmbH
     * @return mixed
     * 
     */
    public function validatePayment() {
        $paymentId = oxRegistry::getConfig()->getRequestParameter('paymentid');
        $parentResult = parent::validatePayment();
        $result['msg'] = '';

        // get current language
        $lang = strtoupper(oxRegistry::getLang()->getLanguageAbbr());

        if ($paymentId == "gc_creditcard") {
            if (oxRegistry::getConfig()->getConfigParam('GC_CREDITCARD_MERCHANTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_CREDITCARD_PROJECTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_CREDITCARD_SECRET') == '') {

                $result['msg'] = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $lang);
            }
        }

        if ($paymentId == "gc_directdebit") {

            if (oxRegistry::getConfig()->getConfigParam('GC_DIRECTDEBIT_MERCHANTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_DIRECTDEBIT_PROJECTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_DIRECTDEBIT_SECRET') == '') {

                $result['msg'] = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $lang);
            }
        }

        if ($paymentId == "gc_eps") {

            if (oxRegistry::getConfig()->getConfigParam('GC_EPS_MERCHANTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_EPS_PROJECTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_EPS_SECRET') == '') {

                $result['msg'] = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $lang);

            }
        }

        if ($paymentId == "gc_giropay") {

            if (oxRegistry::getConfig()->getConfigParam('GC_GIROPAY_MERCHANTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_GIROPAY_PROJECTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_GIROPAY_SECRET') == '') {

                $result['msg'] = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $lang);
            }
        }

        if ($paymentId == "gc_ideal") {

            if (oxRegistry::getConfig()->getConfigParam('GC_IDEAL_MERCHANTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_IDEAL_PROJECTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_IDEAL_SECRET') == '') {

                $result['msg'] = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $lang);
            }
        }

        if ($paymentId == "gc_paydirekt") {
            if (oxRegistry::getConfig()->getConfigParam('GC_PAYDIREKT_MERCHANTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_PAYDIREKT_PROJECTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_PAYDIREKT_SECRET') == '') {

                $result['msg'] = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $lang);
            }
        }

        if ($paymentId == "gc_sofortuw") {
            if (oxRegistry::getConfig()->getConfigParam('GC_SOFORTUW_MERCHANTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_SOFORTUW_PROJECTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_SOFORTUW_SECRET') == '') {

                $result['msg'] = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $lang);
            }
        }

        if (!empty($result['msg'])) {
            $this->_sPaymentError = 'girocheckout';
            $this->_girocheckout_payment_error = $result['msg'];

            return;
        }

        return $parentResult;
    }


    /**
     * return the logo to display for credit card payment method.
     *
     * @author S-Public Services GmbH
     * @package GiroCheckout
     * @copyright Copyright (c) 2016, S-Public Services GmbH
     * @return string
     * 
     */
    public function getGiroCheckoutCreditCardLogo() {
        // set special payment logo
        $visa_msc = oxRegistry::getConfig()->getConfigParam('GC_CREDITCARD_SHOWVISAMC');
        $amex = oxRegistry::getConfig()->getConfigParam('GC_CREDITCARD_SHOWAMEX');
        $jcb = oxRegistry::getConfig()->getConfigParam('GC_CREDITCARD_SHOWJCB');
        if (!$visa_msc && !$amex && !$jcb) {
            $visa_msc = TRUE;
        }
        $sCreditCardLogo = GiroCheckout_SDK_Tools::getCreditCardLogoName($visa_msc, $amex, $jcb, false, 40);

        return $sCreditCardLogo;
    }


    /**
     * return active currency name.
     *
     * @author S-Public Services GmbH
     * @package GiroCheckout
     * @copyright Copyright (c) 2016, S-Public Services GmbH
     * @return string
     * 
     */
    public function getActiveCurrencyName() {
        $oCur = $this->getConfig()->getActShopCurrencyObject();
        $sCur = $oCur->name;

        return $sCur;
    }

    /**
     * check if credentials are set in config for GiroCheckout payment methods.
     *
     * @author S-Public Services GmbH
     * @package GiroCheckout
     * @copyright Copyright (c) 2016, S-Public Services GmbH
     * @return boolean
     * 
     */
    public function isSettingsSet($paymentId) {
        $settingsAreSet = true;

        if ($paymentId == "gc_giropay") {

            if (oxRegistry::getConfig()->getConfigParam('GC_GIROPAY_MERCHANTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_GIROPAY_PROJECTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_GIROPAY_SECRET') == '') {
                $settingsAreSet = false;
            }
        }
        if ($paymentId == "gc_directdebit") {

            if (oxRegistry::getConfig()->getConfigParam('GC_DIRECTDEBIT_MERCHANTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_DIRECTDEBIT_PROJECTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_DIRECTDEBIT_SECRET') == '') {
                $settingsAreSet = false;
            }
        }
        if ($paymentId == "gc_ideal") {

            if (oxRegistry::getConfig()->getConfigParam('GC_IDEAL_MERCHANTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_IDEAL_PROJECTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_IDEAL_SECRET') == '') {
                $settingsAreSet = false;
            }
        }
        if ($paymentId == "gc_creditcard") {

            if (oxRegistry::getConfig()->getConfigParam('GC_CREDITCARD_MERCHANTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_CREDITCARD_PROJECTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_CREDITCARD_SECRET') == '') {
                $settingsAreSet = false;
            }
        }
        if ($paymentId == "gc_eps") {

            if (oxRegistry::getConfig()->getConfigParam('GC_EPS_MERCHANTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_EPS_PROJECTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_EPS_SECRET') == '') {
                $settingsAreSet = false;
            }
        }

        if ($paymentId == "gc_paydirekt") {

            if (oxRegistry::getConfig()->getConfigParam('GC_PAYDIREKT_MERCHANTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_PAYDIREKT_PROJECTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_PAYDIREKT_SECRET') == '') {
                $settingsAreSet = false;
            }
        }

        if ($paymentId == "gc_sofortuw") {

            if (oxRegistry::getConfig()->getConfigParam('GC_SOFORTUW_MERCHANTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_SOFORTUW_PROJECTID') == '' || oxRegistry::getConfig()->getConfigParam('GC_SOFORTUW_SECRET') == '') {
                $settingsAreSet = false;
            }
        }

        return $settingsAreSet;
    }

    /**
     * return source id param.
     *
     * @author S-Public Services GmbH
     * @package GiroCheckout
     * @copyright Copyright (c) 2016, S-Public Services GmbH
     * @return string
     * 
     */
    public function getGcSource() {
        /** @var oxModule $modul */
        $modul = oxNew('oxModule');
        $modul->load('girocheckout');

        return "Oxid " . oxRegistry::getConfig()->getVersion() . ";Oxid Plugin " . $modul->getInfo('version');
    }

}
